@@ -166,20 +166,17 @@ gog_graph_type_name (GogObject const *gobj)
 static void
 cb_theme_changed (GtkComboBox *combo, GogGraph *graph)
 {
-	GSList *theme_names;
-	char const *name;
-	int index = gtk_combo_box_get_active (combo);
+	GtkTreeIter iter;
 
-	theme_names = gog_theme_registry_get_theme_names ();
-	if (theme_names == NULL)
-		return;
-
-	name = g_slist_nth_data (theme_names, index);
-	g_slist_free (theme_names);
-	if (name == NULL)
-		return;
-
-	gog_graph_set_theme (graph, gog_theme_registry_lookup (name));
+	if (gtk_combo_box_get_active_iter (combo, &iter)) {
+		GtkTreeModel *model = GTK_TREE_MODEL (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)));
+		GogTheme *theme = NULL;
+		gtk_tree_model_get (model, &iter, 1, &theme, -1);
+		if (theme != NULL) {
+			gog_graph_set_theme (graph, theme);
+			g_object_unref (theme);
+		}
+	}
 }
 
 static void
@@ -211,16 +208,24 @@ gog_graph_populate_editor (GogObject *gobj,
 		GtkWidget *box;
 		GtkWidget *combo;
 		GSList *ptr;
+		GtkListStore *model;
+		GtkTreeIter iter;
+		GogTheme *theme;
 		char const *graph_theme_name;
 		int count, index = 0;
 
 		graph_theme_name = gog_theme_get_name (graph->theme);
 		combo = go_gtk_builder_get_widget (gui, "theme_combo");
-		gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (combo))));
+		model = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)));
 
 		count = 0;
 		for (ptr = theme_names; ptr != NULL; ptr = ptr->next) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(ptr->data));
+			theme = gog_theme_registry_lookup (ptr->data);
+			gtk_list_store_append (model, &iter);
+			gtk_list_store_set (model, &iter,
+			                    0, gog_theme_get_local_name (theme),
+			                    1, theme,
+			                    -1);
 			if (strcmp (ptr->data, graph_theme_name) == 0)
 				index = count;
 			count++;
@@ -246,9 +251,24 @@ static void
 role_chart_post_add (GogObject *parent, GogObject *chart)
 {
 	GogGraph *graph = GOG_GRAPH (parent);
+	unsigned ypos = 0;
+	if (graph->charts != NULL) {
+		/* find the first row with an unoccupied first column */
+		gboolean *occ = g_alloca (graph->num_rows * sizeof (gboolean));
+		GSList *ptr;
+		int col, row;
+		memset (occ, 0, graph->num_rows * sizeof (gboolean));
+		for (ptr = graph->charts; ptr != NULL; ptr = ptr->next)
+			if (gog_chart_get_position (GOG_CHART (ptr->data), &col, &row, NULL, NULL) && col == 0 && row < (int) graph->num_rows)
+				occ[row] = TRUE;
+		while (ypos < graph->num_rows && occ[ypos])
+			ypos++;
+	}
 	graph->charts = g_slist_prepend (graph->charts, chart);
-	gog_chart_set_position (GOG_CHART (chart),
-		0, GOG_GRAPH (graph)->num_rows, 1, 1);
+	if (!gog_chart_get_position (GOG_CHART (chart), NULL, NULL, NULL, NULL))
+		/* search the first row with column 0 unocupied */
+		gog_chart_set_position (GOG_CHART (chart),
+			0, ypos, 1, 1);
 }
 
 static void
@@ -278,12 +298,12 @@ gog_graph_class_init (GogGraphClass *klass)
 	GogObjectClass *gog_klass = (GogObjectClass *) klass;
 
 	static GogObjectRole const roles[] = {
-		{ N_("Chart"), "GogChart",	0,
+		{ N_("Chart"), "GogChart",	1,
 		  GOG_POSITION_SPECIAL|GOG_POSITION_ANY_MANUAL,
 		  GOG_POSITION_SPECIAL,
 		  GOG_OBJECT_NAME_BY_ROLE,
 		  NULL, NULL, NULL, role_chart_post_add, role_chart_pre_remove, NULL },
-		{ N_("Title"), "GogLabel",	1,
+		{ N_("Title"), "GogLabel",	0,
 		  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL,
 		  GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER,
 		  GOG_OBJECT_NAME_BY_ROLE,
@@ -357,7 +377,7 @@ gog_graph_class_init (GogGraphClass *klass)
 	g_object_class_install_property (gobject_klass, GRAPH_PROP_HEIGHT,
 		g_param_spec_double ("height-pts",
 			_("Height"),
-			_("Logical graph heigth, in points"),
+			_("Logical graph height, in points"),
 			0.0, G_MAXDOUBLE, GOG_GRAPH_DEFAULT_HEIGHT,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GRAPH_PROP_DOCUMENT,
@@ -418,18 +438,24 @@ gog_graph_validate_chart_layout (GogGraph *graph)
 	/* 1) find the max */
 	max_col = max_row = 0;
 	for (ptr = graph->charts ; ptr != NULL ; ptr = ptr->next) {
+		if (gog_object_get_position_flags (ptr->data, GOG_POSITION_MANUAL))
+			continue;
 		chart = ptr->data;
-		if (max_col < (chart->x + chart->cols))
-			max_col = (chart->x + chart->cols);
-		if (max_row < (chart->y + chart->rows))
-			max_row = (chart->y + chart->rows);
+		chart->x_pos_actual = chart->x_pos;
+		chart->y_pos_actual = chart->y_pos;
+		if (max_col < (chart->x_pos_actual + chart->cols))
+			max_col = (chart->x_pos_actual + chart->cols);
+		if (max_row < (chart->y_pos_actual + chart->rows))
+			max_row = (chart->y_pos_actual + chart->rows);
 	}
 
 	/* 2) see if we need to contract any cols */
 	for (i = 0 ; i < max_col ; ) {
 		for (ptr = graph->charts ; ptr != NULL ; ptr = ptr->next) {
+			if (gog_object_get_position_flags (ptr->data, GOG_POSITION_MANUAL))
+				continue;
 			chart = ptr->data;
-			if (chart->x <= i && i < (chart->x + chart->cols))
+			if (chart->x_pos_actual <= i && i < (chart->x_pos_actual + chart->cols))
 				break;
 		}
 		if (ptr == NULL) {
@@ -437,18 +463,20 @@ gog_graph_validate_chart_layout (GogGraph *graph)
 			max_col--;
 			for (ptr = graph->charts ; ptr != NULL ; ptr = ptr->next) {
 				chart = ptr->data;
-				if (chart->x > i)
-					(chart->x)--;
+				if (chart->x_pos_actual > i)
+					(chart->x_pos_actual)--;
 			}
 		} else
-			i = chart->x + chart->cols;
+			i = chart->x_pos_actual + chart->cols;
 	}
 
 	/* 3) see if we need to contract any rows */
 	for (i = 0 ; i < max_row ; ) {
 		for (ptr = graph->charts ; ptr != NULL ; ptr = ptr->next) {
+			if (gog_object_get_position_flags (ptr->data, GOG_POSITION_MANUAL))
+				continue;
 			chart = ptr->data;
-			if (chart->y <= i && i < (chart->y + chart->rows))
+			if (chart->y_pos_actual <= i && i < (chart->y_pos_actual + chart->rows))
 				break;
 		}
 		if (ptr == NULL) {
@@ -456,11 +484,11 @@ gog_graph_validate_chart_layout (GogGraph *graph)
 			max_row--;
 			for (ptr = graph->charts ; ptr != NULL ; ptr = ptr->next) {
 				chart = ptr->data;
-				if (chart->y > i)
-					(chart->y)--;
+				if (chart->y_pos_actual > i)
+					(chart->y_pos_actual)--;
 			}
 		} else
-			i = chart->y + chart->rows;
+			i = chart->y_pos_actual + chart->rows;
 	}
 	changed |= (graph->num_cols != max_col || graph->num_rows != max_row);
 
@@ -797,7 +825,6 @@ gog_graph_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 {
 	GSList *ptr;
 	double h, w;
-	unsigned x, y, rows, cols;
 	GogView *child;
 	GogGraph *graph = GOG_GRAPH (view->model);
 	GogViewAllocation tmp, res = *bbox;
@@ -814,12 +841,11 @@ gog_graph_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		if (GOG_POSITION_IS_SPECIAL (child->model->position)) {
-			gog_chart_get_position (GOG_CHART (child->model),
-				&x, &y, &cols, &rows);
-			tmp.x = x * w + res.x;
-			tmp.y = y * h + res.y;
-			tmp.w = cols * w;
-			tmp.h = rows * h;
+			GogChart *chart = GOG_CHART (child->model);
+			tmp.x = chart->x_pos_actual * w + res.x;
+			tmp.y = chart->y_pos_actual * h + res.y;
+			tmp.w = chart->cols * w;
+			tmp.h = chart->rows * h;
 			gog_view_size_allocate (child, &tmp);
 		}
 	}
@@ -829,9 +855,26 @@ static void
 gog_graph_view_render  (GogView *view, GogViewAllocation const *bbox)
 {
 	GogGraphView *gview = GOG_GRAPH_VIEW (view);
+	GSList *ptr;
+	GogView *child_view;
 
 	gview_parent_klass->render (view, bbox);
 
+	/* render everything except labels */
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+		child_view = ptr->data;
+		if (GOG_IS_LABEL (child_view->model))
+			continue;
+		gog_view_render	(ptr->data, bbox);
+	}
+	/* render labels */
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+		child_view = ptr->data;
+		if (!GOG_IS_LABEL (child_view->model))
+			continue;
+		gog_view_render	(ptr->data, bbox);
+	}
+
 	if (gview->selected_view != NULL)
 		gog_view_render_toolkit (gview->selected_view);
 }
@@ -854,6 +897,7 @@ gog_graph_view_class_init (GogGraphViewClass *gview_klass)
 {
 	GogViewClass *view_klass    = (GogViewClass *) gview_klass;
 	GObjectClass *gobject_klass = (GObjectClass *) view_klass;
+	GogOutlinedViewClass *oview_klass = (GogOutlinedViewClass *) gview_klass;
 
 	gview_parent_klass = g_type_class_peek_parent (gview_klass);
 	gobject_klass->set_property = gog_graph_view_set_property;
@@ -862,6 +906,7 @@ gog_graph_view_class_init (GogGraphViewClass *gview_klass)
 	view_klass->render	    = gog_graph_view_render;
 	view_klass->size_allocate   = gog_graph_view_size_allocate;
 	view_klass->clip	    = TRUE;
+	oview_klass->call_parent_render = FALSE;
 
 	g_object_class_install_property (gobject_klass, GRAPH_VIEW_PROP_RENDERER,
 		g_param_spec_object ("renderer",
